@@ -59,11 +59,11 @@ class MongoDBMemory:
         self.collection_name = self.vector_store_config.get("collection", "knowledge_base")
         self.use_vector_search = self.vector_store_config.get("use_vector_search", True)
         
+        # Initialize embedding model before MongoDB to ensure embedding_model_name is available
+        self._init_embedding_model()
+        
         # Initialize MongoDB client
         self._init_mongodb()
-        
-        # Initialize embedding model
-        self._init_embedding_model()
     
     def _init_mongodb(self):
         """Initialize MongoDB client and collection."""
@@ -159,7 +159,14 @@ class MongoDBMemory:
                 }
             }
             
-            self.collection.create_search_index(vector_index_def, "vector_index")
+            # Use SearchIndexModel for PyMongo 4.6+ compatibility
+            try:
+                from pymongo.operations import SearchIndexModel
+                search_index_model = SearchIndexModel(definition=vector_index_def, name="vector_index")
+                self.collection.create_search_index(search_index_model)
+            except ImportError:
+                # Fallback for older PyMongo versions
+                self.collection.create_search_index(vector_index_def, "vector_index")
             
         except Exception as e:
             logging.warning(f"Could not create vector search index: {e}")
